@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: nedit.c,v 1.49.2.1 2003/06/27 17:49:25 tringali Exp $";
+static const char CVSID[] = "$Id: nedit.c,v 1.83 2004/09/02 08:49:56 edg Exp $";
 /*******************************************************************************
 *									       *
 * nedit.c -- Nirvana Editor main program				       *
@@ -8,8 +8,9 @@ static const char CVSID[] = "$Id: nedit.c,v 1.49.2.1 2003/06/27 17:49:25 tringal
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
-* 									       *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+*                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
@@ -55,6 +56,8 @@ static const char CVSID[] = "$Id: nedit.c,v 1.49.2.1 2003/06/27 17:49:25 tringal
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
+#include <limits.h>
 
 #ifndef NO_XMIM
 #include <X11/Xlocale.h>
@@ -63,6 +66,7 @@ static const char CVSID[] = "$Id: nedit.c,v 1.49.2.1 2003/06/27 17:49:25 tringal
 #endif
 #include <X11/Intrinsic.h>
 #include <Xm/Xm.h>
+#include <Xm/XmP.h>
 #if XmVersion >= 1002
 #include <Xm/RepType.h>
 #endif
@@ -89,13 +93,23 @@ static void unmaskArgvKeywords(int argc, char **argv, const char **maskArgs);
 static void patchResourcesForVisual(void);
 static void patchResourcesForKDEbug(void);
 static void patchLocaleForMotif(void);
+static unsigned char* sanitizeVirtualKeyBindings();
+static int sortAlphabetical(const void* k1, const void* k2);
+static int virtKeyBindingsAreInvalid(const unsigned char* bindings);
+static void restoreInsaneVirtualKeyBindings(unsigned char* bindings);
+#ifdef LESSTIF_VERSION
+static void bogusWarningFilter(String);
+#endif
 
 WindowInfo *WindowList = NULL;
 Display *TheDisplay = NULL;
 char *ArgV0 = NULL;
 Boolean IsServer = False;
+Widget TheAppShell;
 
-/* iso8859 appears to be necessary for newer versions of XFree86 that
+/* Reasons for choice of default font qualifications:
+
+   iso8859 appears to be necessary for newer versions of XFree86 that
    default to Unicode encoding, which doesn't quite work with Motif.
    Otherwise Motif puts up garbage (square blocks).
 
@@ -105,21 +119,41 @@ Boolean IsServer = False;
    RedHat 7.3 won't default to '-1' for an encoding, if left with a *,
    and so reverts to "fixed".  Yech. */
 
-#define NEDIT_DEFAULT_FONT      "-*-helvetica-medium-r-normal-*-*-120-*-*-*-iso8859-1"
-#define NEDIT_FIXED_FONT        "-*-courier-medium-r-normal-*-*-120-*-*-*-iso8859-1"
+#define NEDIT_DEFAULT_FONT      "-*-helvetica-medium-r-normal-*-*-120-*-*-*-iso8859-1," \
+                                "-*-helvetica-bold-r-normal-*-*-120-*-*-*-iso8859-1=BOLD," \
+                                "-*-helvetica-medium-o-normal-*-*-120-*-*-*-iso8859-1=ITALIC"
+
+#define NEDIT_FIXED_FONT        "-*-courier-medium-r-normal-*-*-120-*-*-*-iso8859-1," \
+                                "-*-courier-bold-r-normal-*-*-120-*-*-*-iso8859-1=BOLD," \
+                                "-*-courier-medium-o-normal-*-*-120-*-*-*-iso8859-1=ITALIC"
+
 #define NEDIT_DEFAULT_BG        "#b3b3b3"
 
 static char *fallbackResources[] = {
     /* Try to avoid Motif's horrificly ugly default colors and fonts,
        if the user's environment provides no usable defaults.  We try
        to choose a Windows-y default color setting here.  Editable text 
-       fields are forced to a fixed-pitch font for usability. */
+       fields are forced to a fixed-pitch font for usability.
+
+       By using the VendorShell fontList resources, Motif automatically
+       groups the fonts into the right classes.  It's then easier for
+       the user or environment to override this sensibly:
+
+       nedit -xrm '*textFontList: myfont'
+      */
+
+#if (defined(LESSTIF_VERSION) && defined(__CYGWIN__))
     "*FontList: "               NEDIT_DEFAULT_FONT,
     "*XmText.FontList: "        NEDIT_FIXED_FONT,
     "*XmTextField.FontList: "   NEDIT_FIXED_FONT,
     "*XmList.FontList: "        NEDIT_FIXED_FONT,
-    "*XmFileSelectionBox*XmList.FontList: " 
-                                NEDIT_FIXED_FONT,
+    "*XmFileSelectionBox*XmList.FontList: " 	 NEDIT_FIXED_FONT,
+#else
+    "*buttonFontList: "         NEDIT_DEFAULT_FONT,
+    "*labelFontList: "          NEDIT_DEFAULT_FONT,
+    "*textFontList: "           NEDIT_FIXED_FONT,
+#endif
+
     "*background: "             NEDIT_DEFAULT_BG,
     "*foreground: "             NEDIT_DEFAULT_FG,
     "*XmText.foreground: "      NEDIT_DEFAULT_FG,
@@ -140,6 +174,15 @@ static char *fallbackResources[] = {
         "Ctrl~Alt~Meta<KeyPress>x: cut-clipboard()\\n"
         "Ctrl~Alt~Meta<KeyPress>u: delete-to-start-of-line()\\n",
 
+    "*XmLFolder.highlightThickness: 0",
+    "*XmLFolder.shadowThickness:    1",
+    "*XmLFolder.maxTabWidth:        150",
+    "*XmLFolder.traversalOn:        False",
+    "*XmLFolder.inactiveForeground: #666" ,
+    "*tab.alignment: XmALIGNMENT_BEGINNING",
+    "*tab.marginWidth: 0",
+    "*tab.marginHeight: 1",
+
     /* Prevent the file selection box from acting stupid. */
     "*XmFileSelectionBox.resizePolicy: XmRESIZE_NONE",
     "*XmFileSelectionBox.textAccelerators:",
@@ -168,7 +211,6 @@ static char *fallbackResources[] = {
     "*helpText.selectBackground: " NEDIT_DEFAULT_BG,
     "*statsLine.background: " NEDIT_DEFAULT_BG,
     "*statsLine.FontList: " NEDIT_DEFAULT_FONT,
-    "*helpText.font: " NEDIT_FIXED_FONT,
     "*calltip.background: LemonChiffon1",
     "*calltip.foreground: black",
     "*iSearchForm*highlightThickness: 1",
@@ -183,6 +225,8 @@ static char *fallbackResources[] = {
     "*fileMenu.mnemonic: F",
     "*fileMenu.new.accelerator: Ctrl<Key>n",
     "*fileMenu.new.acceleratorText: Ctrl+N",
+    "*fileMenu.newOpposite.accelerator: Shift Ctrl<Key>n",
+    "*fileMenu.newOpposite.acceleratorText: Shift+Ctrl+N",
     "*fileMenu.open.accelerator: Ctrl<Key>o",
     "*fileMenu.open.acceleratorText: Ctrl+O",
     "*fileMenu.openSelected.accelerator: Ctrl<Key>y",
@@ -295,15 +339,15 @@ static char *fallbackResources[] = {
     "*macroMenu.repeat.accelerator: Ctrl<Key>comma",
     "*macroMenu.repeat.acceleratorText: Ctrl+,",
     "*windowsMenu.mnemonic: W",
-    "*windowsMenu.splitWindow.accelerator: Ctrl<Key>2",
-    "*windowsMenu.splitWindow.acceleratorText: Ctrl+2",
+    "*windowsMenu.splitPane.accelerator: Ctrl<Key>2",
+    "*windowsMenu.splitPane.acceleratorText: Ctrl+2",
     "*windowsMenu.closePane.accelerator: Ctrl<Key>1",
     "*windowsMenu.closePane.acceleratorText: Ctrl+1",
     "*helpMenu.mnemonic: H",
-    "nedit.helpForm.sw.helpText*translations: #override\
+    "nedit.help.helpForm.sw.helpText*translations: #override\
 <Key>Tab:help-focus-buttons()\\n\
-<Key>Return:help-button-action(\"dismiss\")\\n\
-<KeyPress>osfCancel:help-button-action(\"dismiss\")\\n\
+<Key>Return:help-button-action(\"close\")\\n\
+<KeyPress>osfCancel:help-button-action(\"close\")\\n\
 ~Meta~Ctrl~Shift<Btn1Down>:\
     grab-focus() help-hyperlink()\\n\
 ~Meta~Ctrl~Shift<Btn1Up>:\
@@ -324,8 +368,8 @@ static const char cmdLineHelp[] =
 	      [-geometry geometry] [-iconic] [-noiconic] [-svrname name]\n\
 	      [-display [host]:server[.screen] [-xrm resourcestring]\n\
 	      [-import file] [-background color] [-foreground color]\n\
-	      [-V|-version] [--]\n\
-	      [file...]\n";
+	      [-tabbed] [-untabbed] [-group] [-V|-version]\n\
+	      [--] [file...]\n";
 #else
 "";
 #endif /*VMS*/
@@ -334,14 +378,16 @@ int main(int argc, char **argv)
 {
     int i, lineNum, nRead, fileSpecified = FALSE, editFlags = CREATE;
     int gotoLine = False, macroFileRead = False, opts = True;
-    int iconic = False;
+    int iconic=False, tabbed = -1, group = 0, isTabbed;
     char *toDoCommand = NULL, *geometry = NULL, *langMode = NULL;
     char filename[MAXPATHLEN], pathname[MAXPATHLEN];
     XtAppContext context;
     XrmDatabase prefDB;
+    WindowInfo *window = NULL, *lastFile = NULL;
     static const char *protectedKeywords[] = {"-iconic", "-icon", "-geometry",
             "-g", "-rv", "-reverse", "-bd", "-bordercolor", "-borderwidth",
 	    "-bw", "-title", NULL};
+    unsigned char* invalidBindings = NULL;
     
     /* Save the command which was used to invoke nedit for restart command */
     ArgV0 = argv[0];
@@ -357,6 +403,11 @@ int main(int argc, char **argv)
     /* Set up a warning handler to trap obnoxious Xt grab warnings */
     SuppressPassiveGrabWarnings();
 
+#ifdef LESSTIF_VERSION
+    /* Set up a handler to suppress bogus Lesstif warning messages */
+    XtAppSetWarningHandler(context, bogusWarningFilter);
+#endif
+    
     /* Set up default resources if no app-defaults file is found */
     XtAppSetFallbackResources(context, fallbackResources);
     
@@ -387,10 +438,20 @@ int main(int argc, char **argv)
 	    NULL, 0, &argc, argv);
     unmaskArgvKeywords(argc, argv, protectedKeywords);
     if (!TheDisplay) {
+        /* Respond to -V or -version even if there is no display */
+        for (i = 1; i < argc && strcmp(argv[i], "--"); i++)
+        {
+            if (0 == strcmp(argv[i], "-V") || 0 == strcmp(argv[i], "-version"))
+            {
+                PrintVersion();
+                exit(EXIT_SUCCESS);
+            }
+        }
 	XtWarning ("NEdit: Can't open display\n");
 	exit(EXIT_FAILURE);
     }
-
+    
+    /* Must be done before creating widgets */
     patchResourcesForVisual();
     patchResourcesForKDEbug();
     
@@ -401,6 +462,32 @@ int main(int argc, char **argv)
     /* Store preferences from the command line and .nedit file, 
        and set the appropriate preferences */
     RestoreNEditPrefs(prefDB, XtDatabase(TheDisplay));
+
+    /* Intercept syntactically invalid virtual key bindings BEFORE we 
+       create any shells. */
+    invalidBindings = sanitizeVirtualKeyBindings();
+    
+    /* Create a hidden application shell that is the parent of all the
+       main editor windows.  Realize it so it the window can act as 
+       group leader. */
+    TheAppShell = CreateShellWithBestVis(APP_NAME, 
+                                         APP_CLASS,
+                                         applicationShellWidgetClass,
+                                         TheDisplay,
+                                         NULL,
+                                         0);
+    
+    /* Restore the original bindings ASAP such that other apps are not affected. */
+    restoreInsaneVirtualKeyBindings(invalidBindings);
+
+    XtSetMappedWhenManaged(TheAppShell, False);
+    XtRealizeWidget(TheAppShell);
+
+#ifndef NO_SESSION_RESTART
+    AttachSessionMgrHandler(TheAppShell);
+#endif
+
+    /* More preference stuff */
     LoadPrintPreferences(XtDatabase(TheDisplay), APP_NAME, APP_CLASS, True);
     SetDeleteRemap(GetPrefMapDelete());
     SetPointerCenteredDialogs(GetPrefRepositionDialogs());
@@ -440,8 +527,13 @@ int main(int argc, char **argv)
     /* Load the default tags file. Don't complain if it doesn't load, the tag
        file resource is intended to be set and forgotten.  Running nedit in a
        directory without a tags should not cause it to spew out errors. */
-    if (*GetPrefTagFile() != '\0')
-    	AddTagsFile(GetPrefTagFile(), TAG);
+    if (*GetPrefTagFile() != '\0') {
+        AddTagsFile(GetPrefTagFile(), TAG);
+    }
+
+    if (strcmp(GetPrefServerName(), "") != 0) {
+        IsServer = True;
+    }
 
     /* Process any command line arguments (-tags, -do, -read, -create,
        +<line_number>, -line, -server, and files to edit) not already
@@ -463,6 +555,14 @@ int main(int argc, char **argv)
     	    editFlags |= PREF_READ_ONLY;
     	} else if (opts && !strcmp(argv[i], "-create")) {
     	    editFlags |= SUPPRESS_CREATE_WARN;
+    	} else if (opts && !strcmp(argv[i], "-tabbed")) {
+    	    tabbed = 1;
+    	    group = 0;	/* override -group option */
+    	} else if (opts && !strcmp(argv[i], "-untabbed")) {
+    	    tabbed = 0;
+    	    group = 0;	/* override -group option */
+    	} else if (opts && !strcmp(argv[i], "-group")) {
+    	    group = 2; /* 2: start new group, 1: in group */
     	} else if (opts && !strcmp(argv[i], "-line")) {
     	    nextArg(argc, argv, &i);
 	    nRead = sscanf(argv[i], "%d", &lineNum);
@@ -513,17 +613,56 @@ int main(int argc, char **argv)
 	    /* for each expanded file name do: */
 	    for (j = 0; j < numFiles; ++j) {
 	    	if (ParseFilename(nameList[j], filename, pathname) == 0) {
-		    EditExistingFile(WindowList, filename, pathname, editFlags,
-				     geometry, iconic, langMode);
-		    fileSpecified = TRUE;
-		    if (!macroFileRead) {
-		        ReadMacroInitFile(WindowList);
-		        macroFileRead = True;
+		    /* determine if file is to be openned in new tab, by
+		       factoring the options -group, -tabbed & -untabbed */
+    		    if (group == 2) {
+	        	isTabbed = 0;  /* start a new window for new group */
+			group = 1;     /* next file will be within group */
+		    } else if (group == 1) {
+	    		isTabbed = 1;  /* new tab for file in group */
+		    } else {           /* not in group */
+	    		isTabbed = tabbed==-1? GetPrefOpenInTab() : tabbed; 
 		    }
-		    if (toDoCommand != NULL)
-	    	        DoMacro(WindowList, toDoCommand, "-do macro");
-	    	    if (gotoLine)
-	    	        SelectNumberedLine(WindowList, lineNum);
+
+		    /* Files are opened in background to improve opening speed
+		       by defering certain time  consuiming task such as syntax
+		       highlighting. At the end of the file-opening loop, the 
+		       last file opened will be raised to restore those deferred
+		       items. The current file may also be raised if there're
+		       macros to execute on. */
+		    window = EditExistingFile(WindowList, filename, pathname, 
+		    	    editFlags, geometry, iconic, langMode, isTabbed, 
+			    True);
+		    fileSpecified = TRUE;
+
+		    if (window) {
+			CleanUpTabBarExposeQueue(window);
+
+			/* raise the last file of previous window */
+			if (lastFile && window->shell != lastFile->shell) {
+			    CleanUpTabBarExposeQueue(lastFile);
+			    RaiseDocument(lastFile);
+			}
+			
+			if (!macroFileRead) {
+		            ReadMacroInitFile(WindowList);
+		            macroFileRead = True;
+			}
+	    		if (gotoLine)
+	    	            SelectNumberedLine(window, lineNum);
+			if (toDoCommand != NULL) {
+			    DoMacro(window, toDoCommand, "-do macro");
+			    toDoCommand = NULL;
+			    if (!IsValidWindow(window))
+		    		window = NULL; /* window closed by macro */
+			    if (lastFile && !IsValidWindow(lastFile))
+		    		lastFile = NULL; /* window closed by macro */
+			}
+		    }
+		    
+		    /* register last opened file for later use */
+		    if (window)
+    	    		lastFile = window;
                 } else {
 		    fprintf(stderr, "nedit: file name too long: %s\n", nameList[j]);
                 }
@@ -533,32 +672,76 @@ int main(int argc, char **argv)
 	    	free(nameList);
 #else
 	    if (ParseFilename(argv[i], filename, pathname) == 0 ) {
-		EditExistingFile(WindowList, filename, pathname, editFlags,
-				 geometry, iconic, langMode);
-		fileSpecified = TRUE;
-		if (!macroFileRead) {
-		    ReadMacroInitFile(WindowList);
-		    macroFileRead = True;
+		/* determine if file is to be openned in new tab, by
+		   factoring the options -group, -tabbed & -untabbed */
+    		if (group == 2) {
+	            isTabbed = 0;  /* start a new window for new group */
+		    group = 1;     /* next file will be within group */
+		} else if (group == 1) {
+	    	    isTabbed = 1;  /* new tab for file in group */
+		} else {           /* not in group */
+	    	    isTabbed = tabbed==-1? GetPrefOpenInTab() : tabbed; 
 		}
-		if (toDoCommand != NULL)
-		    DoMacro(WindowList, toDoCommand, "-do macro");
-		if (gotoLine)
-		    SelectNumberedLine(WindowList, lineNum);
+		
+		/* Files are opened in background to improve opening speed
+		   by defering certain time  consuiming task such as syntax
+		   highlighting. At the end of the file-opening loop, the 
+		   last file opened will be raised to restore those deferred
+		   items. The current file may also be raised if there're
+		   macros to execute on. */
+		window = EditExistingFile(WindowList, filename, pathname, 
+		    	editFlags, geometry, iconic, langMode, isTabbed, True);
+    	    	fileSpecified = TRUE;
+		if (window) {
+		    CleanUpTabBarExposeQueue(window);
+
+		    /* raise the last tab of previous window */
+		    if (lastFile && window->shell != lastFile->shell) {
+			CleanUpTabBarExposeQueue(lastFile);
+			RaiseDocument(lastFile);
+		    }
+
+		    if (!macroFileRead) {
+			ReadMacroInitFile(WindowList);
+			macroFileRead = True;
+		    }
+		    if (gotoLine)
+			SelectNumberedLine(window, lineNum);
+		    if (toDoCommand != NULL) {
+			DoMacro(window, toDoCommand, "-do macro");
+	    	    	toDoCommand = NULL;
+			if (!IsValidWindow(window))
+		    	    window = NULL; /* window closed by macro */
+			if (lastFile && !IsValidWindow(lastFile))
+		    	    lastFile = NULL; /* window closed by macro */
+		    }
+		}
+		
+		/* register last opened file for later use */
+		if (window)
+    	    	    lastFile = window;
 	    } else {
 		fprintf(stderr, "nedit: file name too long: %s\n", argv[i]);
 	    }
 #endif /*VMS*/
-	    toDoCommand = NULL;
 	}
     }
 #ifdef VMS
     VMSFileScanDone();
 #endif /*VMS*/
     
+    /* Raise the last file opened */
+    if (lastFile) {
+	CleanUpTabBarExposeQueue(lastFile);
+	RaiseDocument(lastFile);
+    }
+    CheckCloseDim();
+
     /* If no file to edit was specified, open a window to edit "Untitled" */
     if (!fileSpecified) {
-    	EditNewFile(geometry, iconic, langMode, NULL);
+    	EditNewFile(NULL, geometry, iconic, langMode, NULL);
 	ReadMacroInitFile(WindowList);
+	CheckCloseDim();
 	if (toDoCommand != NULL)
 	    DoMacro(WindowList, toDoCommand, "-do macro");
     }
@@ -851,3 +1034,189 @@ static String neditLanguageProc(Display *dpy, String xnl, XtPointer closure)
 
     return setlocale(LC_ALL, NULL); /* re-query in case overwritten */
 }
+
+static int sortAlphabetical(const void* k1, const void* k2)
+{
+    const char* key1 = *(const char**)k1;
+    const char* key2 = *(const char**)k2;
+    return strcmp(key1, key2);
+}
+
+/*
+ * Checks whether a given virtual key binding string is invalid. 
+ * A binding is considered invalid if there are duplicate key entries.
+ */
+static int virtKeyBindingsAreInvalid(const unsigned char* bindings)
+{
+    int maxCount = 1, i, count;
+    const char  *pos = (const char*)bindings;
+    char *copy;
+    char *pos2, *pos3;
+    char **keys;
+
+    /* First count the number of bindings; bindings are separated by \n
+       strings. The number of bindings equals the number of \n + 1.
+       Beware of leading and trailing \n; the number is actually an
+       upper bound on the number of entries. */
+    while ((pos = strstr(pos, "\n"))) 
+    { 
+        ++pos; 
+        ++maxCount;
+    }
+    
+    if (maxCount == 1) 
+        return False; /* One binding is always ok */
+    
+    keys = (char**)malloc(maxCount*sizeof(char*));
+    copy = XtNewString((const char*)bindings);
+    i = 0;
+    pos2 = copy;
+    
+    count = 0;
+    while (i<maxCount && pos2 && *pos2)
+    {
+        while (isspace((int) *pos2) || *pos2 == '\n') ++pos2;
+        
+        if (*pos2 == '!') /* Ignore comment lines */
+        {
+            pos2 = strstr(pos2, "\n");
+            continue; /* Go to the next line */
+        }
+
+        if (*pos2)
+        {
+            keys[i++] = pos2;
+            ++count;
+            pos3 = strstr(pos2, ":");
+            if (pos3) 
+            {
+                *pos3++ = 0; /* Cut the string and jump to the next entry */
+                pos2 = pos3;
+            }
+            pos2 = strstr(pos2, "\n");
+        }
+    }
+    
+    if (count <= 1)
+    {
+       free(keys);
+       XtFree(copy);
+       return False; /* No conflict */
+    }
+    
+    /* Sort the keys and look for duplicates */
+    qsort((void*)keys, count, sizeof(const char*), sortAlphabetical);
+    for (i=1; i<count; ++i)
+    {
+        if (!strcmp(keys[i-1], keys[i]))
+        {
+            /* Duplicate detected */
+            free(keys);
+            XtFree(copy);
+            return True;
+        }
+    }
+    free(keys);
+    XtFree(copy);
+    return False;
+}
+
+/*
+ * Optionally sanitizes the Motif default virtual key bindings. 
+ * Some applications install invalid bindings (attached to the root window),
+ * which cause certain keys to malfunction in NEdit.
+ * Through an X-resource, users can choose whether they want
+ *   - to always keep the existing bindings
+ *   - to override the bindings only if they are invalid
+ *   - to always override the existing bindings.
+ */
+
+static Atom virtKeyAtom;
+
+static unsigned char* sanitizeVirtualKeyBindings()
+{
+    int overrideBindings = GetPrefOverrideVirtKeyBindings();
+    Window rootWindow;
+    const char *virtKeyPropName = "_MOTIF_DEFAULT_BINDINGS";
+    Atom dummyAtom;
+    int getFmt;
+    unsigned long dummyULong, nItems;
+    unsigned char *insaneVirtKeyBindings = NULL;
+    
+    if (overrideBindings == VIRT_KEY_OVERRIDE_NEVER) return NULL;
+    
+    virtKeyAtom =  XInternAtom(TheDisplay, virtKeyPropName, False);
+    rootWindow = RootWindow(TheDisplay, DefaultScreen(TheDisplay));
+
+    /* Remove the property, if it exists; we'll restore it later again */
+    if (XGetWindowProperty(TheDisplay, rootWindow, virtKeyAtom, 0, INT_MAX, 
+                           True, XA_STRING, &dummyAtom, &getFmt, &nItems, 
+                           &dummyULong, &insaneVirtKeyBindings) != Success 
+        || nItems == 0) 
+    {
+        return NULL; /* No binding yet; nothing to do */
+    }
+    
+    if (overrideBindings == VIRT_KEY_OVERRIDE_AUTO)
+    {   
+        if (!virtKeyBindingsAreInvalid(insaneVirtKeyBindings))
+        {
+            /* Restore the property immediately; it seems valid */
+            XChangeProperty(TheDisplay, rootWindow, virtKeyAtom, XA_STRING, 8,
+                            PropModeReplace, insaneVirtKeyBindings, 
+                            strlen((const char*)insaneVirtKeyBindings));
+            XFree((char*)insaneVirtKeyBindings);
+            return NULL; /* Prevent restoration */
+        }
+    }
+    return insaneVirtKeyBindings;
+}
+
+/*
+ * NEdit should not mess with the bindings installed by other apps, so we
+ * just restore whatever was installed, if necessary
+ */
+static void restoreInsaneVirtualKeyBindings(unsigned char *insaneVirtKeyBindings)
+{
+   if (insaneVirtKeyBindings)
+   {
+      Window rootWindow = RootWindow(TheDisplay, DefaultScreen(TheDisplay));
+      /* Restore the root window atom, such that we don't affect 
+         other apps. */
+      XChangeProperty(TheDisplay, rootWindow, virtKeyAtom, XA_STRING, 8,
+                      PropModeReplace, insaneVirtKeyBindings, 
+                      strlen((const char*)insaneVirtKeyBindings));
+      XFree((char*)insaneVirtKeyBindings);
+   }
+}
+
+#ifdef LESSTIF_VERSION
+/*
+** Warning handler that suppresses harmless but annoying warnings generated
+** by non-production Lesstif versions.
+*/
+static void bogusWarningFilter(String message)
+{
+  const char* bogusMessages[] = {
+    "XmFontListCreate() is an obsolete function!",
+    "No type converter registered for 'String' to 'PathMode' conversion.",
+    "XtRemoveGrab asked to remove a widget not on the list",
+    NULL };
+  const char **bogusMessage = &bogusMessages[0]; 
+  
+  while (*bogusMessage) {
+      size_t bogusLen = strlen(*bogusMessage);
+      if (strncmp(message, *bogusMessage, bogusLen) == 0) {
+#ifdef DEBUG_LESSTIF_WARNINGS
+         /* Developers may want to see which messages are suppressed. */
+         fprintf(stderr, "[SUPPRESSED] %s\n", message);
+#endif        
+         return;
+      }
+      ++bogusMessage;
+  }
+  
+  /* An unknown message. Keep it. */
+  fprintf(stderr, "%s\n", message);
+}
+#endif /* LESSTIF_VERSION */
