@@ -1,4 +1,4 @@
-/* $Id: rangeset.c,v 1.7.2.5 2003/09/08 17:03:32 edg Exp $ */
+/* $Id: rangeset.c,v 1.11 2004/07/21 11:32:05 yooden Exp $ */
 /*******************************************************************************
 *									       *
 * rangeset.c	 -- Nirvana Editor rangest functions			       *
@@ -8,8 +8,9 @@
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.								       *
-*									       *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+*                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or	       *
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License	       *
@@ -780,6 +781,69 @@ RangesetTable *RangesetTableFree(RangesetTable *table)
     return (RangesetTable *)0;
 }
 
+/*
+** clone a ranges set.
+*/
+static void rangesetClone(Rangeset *destRangeset, Rangeset *srcRangeset)
+{
+    destRangeset->update_fn   = srcRangeset->update_fn;
+    destRangeset->update_name = srcRangeset->update_name;
+    destRangeset->maxpos      = srcRangeset->maxpos;
+    destRangeset->last_index  = srcRangeset->last_index;
+    destRangeset->n_ranges    = srcRangeset->n_ranges;
+    destRangeset->color_set   = srcRangeset->color_set;
+    destRangeset->color       = srcRangeset->color;
+
+    if (srcRangeset->color_name) {
+	destRangeset->color_name = XtMalloc(strlen(srcRangeset->color_name) +1);
+	strcpy(destRangeset->color_name, srcRangeset->color_name);
+    }
+
+    if (srcRangeset->name) {
+	destRangeset->name = XtMalloc(strlen(srcRangeset->name) + 1);
+	strcpy(destRangeset->name, srcRangeset->name);
+    }
+
+    if (srcRangeset->ranges) {
+	destRangeset->ranges = RangesNew(srcRangeset->n_ranges);
+	memcpy(destRangeset->ranges, srcRangeset->ranges,
+		srcRangeset->n_ranges * sizeof(Range));
+    }
+}
+
+/*
+** Create a new rangeset table in the destination buffer
+** by cloning it from the source table.
+**
+** Returns the new table created.
+*/
+RangesetTable *RangesetTableClone(RangesetTable *srcTable,
+        textBuffer *destBuffer)
+{
+    RangesetTable *newTable = NULL;
+    int i;
+
+    if (srcTable == NULL)
+    	return NULL;
+	
+    newTable = RangesetTableAlloc(destBuffer);
+
+    newTable->n_set = srcTable->n_set;
+    memcpy(newTable->order, srcTable->order, 
+	    sizeof(unsigned char) *N_RANGESETS);
+    memcpy(newTable->active, srcTable->active, 
+	    sizeof(unsigned char) *N_RANGESETS);
+    memcpy(newTable->depth, srcTable->depth, 
+	    sizeof(unsigned char) *N_RANGESETS);
+    memcpy(newTable->list, srcTable->list, 
+	    sizeof(unsigned char) *(N_RANGESETS + 1));
+
+    for (i = 0; i < N_RANGESETS; i++)
+	rangesetClone(&newTable->set[i], &srcTable->set[i]);
+    
+    return newTable;
+}
+
 /*
 ** Find a range set given its label in the table.
 */
