@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: textBuf.c,v 1.28.2.1 2003/06/18 09:54:49 edg Exp $";
+static const char CVSID[] = "$Id: textBuf.c,v 1.32 2004/07/21 11:32:05 yooden Exp $";
 /*******************************************************************************
 *                                                                              *
 * textBuf.c - Manage source text for one or more text areas                    *
@@ -8,7 +8,8 @@ static const char CVSID[] = "$Id: textBuf.c,v 1.28.2.1 2003/06/18 09:54:49 edg E
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
@@ -38,8 +39,6 @@ static const char CVSID[] = "$Id: textBuf.c,v 1.28.2.1 2003/06/18 09:54:49 edg E
 #include <string.h>
 #include <ctype.h>
 
-#include <Xm/Xm.h>
-
 #ifdef HAVE_DEBUG_H
 #include "../debug.h"
 #endif
@@ -1097,7 +1096,7 @@ int BufCountDispChars(textBuffer *buf, int lineStartPos, int targetPos)
     char expandedChar[MAX_EXP_CHAR_LEN];
     
     pos = lineStartPos;
-    while (pos < targetPos)
+    while (pos < targetPos && pos < buf->length)
     	charCount += BufGetExpandedChar(buf, pos++, charCount, expandedChar);
     return charCount;
 }
