@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: fontsel.c,v 1.25 2003/05/02 19:19:03 edg Exp $";
+static const char CVSID[] = "$Id: fontsel.c,v 1.28 2004/08/01 10:06:12 yooden Exp $";
 /*******************************************************************************
 *                                                                              *
 * fontsel.c -- Nirvana Font Selector                                           *
@@ -8,7 +8,8 @@ static const char CVSID[] = "$Id: fontsel.c,v 1.25 2003/05/02 19:19:03 edg Exp $
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.                                                                     *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
 *                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
@@ -1138,7 +1139,7 @@ static void choiceMade(xfselControlBlkType *ctrlBlk)
     else
     {
         DialogF (DF_ERR, ctrlBlk->form, 1, "Font Specification",
-                "Invalid Font Specification", "Dismiss");
+                "Invalid Font Specification", "OK");
     }
 }
 
@@ -1208,7 +1209,7 @@ static void okAction(Widget widget, xfselControlBlkType *ctrlBlk,
     if (i != 1)
     {
         DialogF (DF_ERR, ctrlBlk->okButton, 1, "Font Specification",
-                "Invalid Font Specification", "Dismiss");
+                "Invalid Font Specification", "OK");
         XFreeFontNames(fontName);
     }
     else
