@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: managedList.c,v 1.12 2003/05/02 19:19:03 edg Exp $";
+static const char CVSID[] = "$Id: managedList.c,v 1.14 2004/07/21 11:32:07 yooden Exp $";
 /*******************************************************************************
 *									       *
 * managedList.c -- User interface for reorderable list of records	       *
@@ -8,8 +8,9 @@ static const char CVSID[] = "$Id: managedList.c,v 1.12 2003/05/02 19:19:03 edg E
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
-* 									       *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+*                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
@@ -250,7 +251,7 @@ Widget ManageListAndButtons(Widget listW, Widget deleteBtn, Widget copyBtn,
     ml->moveUpBtn = moveUpBtn;
     ml->moveDownBtn = moveDownBtn;
     ml->getDialogDataCB = NULL;
-    ml->getDialogDataArg = setDialogDataArg;
+    ml->getDialogDataArg = getDialogDataArg;
     ml->setDialogDataCB = NULL;
     ml->setDialogDataArg = setDialogDataArg;
     ml->freeItemCB = freeItemCB;
