@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: misc.c,v 1.57.2.1 2003/06/27 07:24:04 edg Exp $";
+static const char CVSID[] = "$Id: misc.c,v 1.69 2004/08/09 22:20:00 n8gray Exp $";
 /*******************************************************************************
 *									       *
 * misc.c -- Miscelaneous Motif convenience functions			       *
@@ -8,8 +8,9 @@ static const char CVSID[] = "$Id: misc.c,v 1.57.2.1 2003/06/27 07:24:04 edg Exp
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
-* 									       *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+*                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
@@ -77,6 +78,10 @@ static const char CVSID[] = "$Id: misc.c,v 1.57.2.1 2003/06/27 07:24:04 edg Exp
 #include "../debug.h"
 #endif
 
+#ifndef LESSTIF_VERSION
+extern void _XmDismissTearOff(Widget w, XtPointer call, XtPointer x);
+#endif
+
 /* structure for passing history-recall data to callbacks */
 typedef struct {
     char ***list;
@@ -146,6 +151,7 @@ static void pageDownAP(Widget w, XEvent *event, String *args,
 	Cardinal *nArgs);
 static void pageUpAP(Widget w, XEvent *event, String *args, 
 	Cardinal *nArgs);
+static long queryDesktop(Display *display, Window window, Atom deskTopAtom);
 
 /*
 ** Set up closeCB to be called when the user selects close from the
@@ -571,6 +577,7 @@ Widget CreateFileSelectionDialog(Widget parent, char *name,
 {
     Widget dialog = addParentVisArgsAndCall(XmCreateFileSelectionDialog, parent, 
             name, arglist, argcount);
+
     AddMouseWheelSupport(XmFileSelectionBoxGetChild(dialog, XmDIALOG_LIST));
     AddMouseWheelSupport(XmFileSelectionBoxGetChild(dialog, XmDIALOG_DIR_LIST));
     return dialog;
@@ -600,8 +607,17 @@ Widget CreateErrorDialog(Widget parent, char *name,
 	    argcount);
 }
 
+Widget CreateWidget(Widget parent, const char *name, WidgetClass class,
+	ArgList arglist, Cardinal  argcount)
+{
+    Widget result;
+    ArgList al = addParentVisArgs(parent, arglist, &argcount);
+    result = XtCreateWidget(name, class, parent, al, argcount);
+    XtFree((char *)al);
+    return result;
+}
 
-Widget CreateShellWithBestVis(String appName, String appClass, 
+Widget CreateShellWithBestVis(String appName, String appClass,
 	   WidgetClass class, Display *display, ArgList args, Cardinal nArgs)
 {
     Visual *visual;
@@ -823,7 +839,8 @@ void RaiseWindow(Display *display, Window w)
 
     XGetWindowAttributes(display, w, &winAttr);
     if (winAttr.map_state == IsViewable)
-    	XSetInputFocus(display, w, RevertToParent, CurrentTime);	
+        XSetInputFocus(display, w, RevertToParent, CurrentTime);	
+
     XMapRaised(display, w);
 }
 
@@ -1214,12 +1231,12 @@ int GetFloatTextWarn(Widget text, double *value, const char *fieldName,
 
     if (result == TEXT_IS_BLANK)
     {
-        DialogF(DF_ERR, text, 1, "Warning", "Please supply %s value", "Dismiss",
+        DialogF(DF_ERR, text, 1, "Warning", "Please supply %s value", "OK",
                 fieldName);
     } else /* TEXT_NOT_NUMBER */
     {
         DialogF (DF_ERR, text, 1, "Warning", "Can't read %s value: \"%s\"",
-                "Dismiss", fieldName, valueStr);
+                "OK", fieldName, valueStr);
     }
 
     XtFree(valueStr);
@@ -1239,11 +1256,11 @@ int GetIntTextWarn(Widget text, int *value, const char *fieldName, int warnBlank
     if (result == TEXT_IS_BLANK)
     {
         DialogF (DF_ERR, text, 1, "Warning", "Please supply a value for %s",
-                "Dismiss", fieldName);
+                "OK", fieldName);
     } else /* TEXT_NOT_NUMBER */
     {
         DialogF (DF_ERR, text, 1, "Warning",
-                "Can't read integer value \"%s\" in %s", "Dismiss", valueStr,
+                "Can't read integer value \"%s\" in %s", "OK", valueStr,
                 fieldName);
     }
 
@@ -2100,17 +2117,21 @@ static void scrollDownAP(Widget w, XEvent *event, String *args, Cardinal *nArgs)
 ** selected button without selection another radio button first.
 ** The workaround consist of faking a mouse click on the button that we 
 ** toggled by calling the Arm, Select, and Disarm action procedures.
+** 
+** A minor remaining issue is the fact that, if the workaround is used, 
+** it is not possible to change the state without notifying potential
+** XmNvalueChangedCallbacks. In practice, this doesn't seem to be a problem.
 **
 */
-void RadioButtonChangeStateNotified(Widget widget, Boolean state)
+void RadioButtonChangeState(Widget widget, Boolean state, Boolean notify)
 {
    /* 
-      The bug only exists in OpenMotif 2.x. Since it's quite hard to detect
-      OpenMotif reliably, we make a rough cut by excluding Lesstif and all
-      Motif versions < 2.x.
+      The bug only exists in OpenMotif 2.1.x/2.2.[0-2]. Since it's quite hard 
+      to detect OpenMotif reliably, we make a rough cut by excluding Lesstif
+      and all Motif versions >= 2.1.x and < 2.2.3.
    */
 #ifndef LESSTIF_VERSION
-#if XmVersion >= 2000
+#if XmVersion == 2001 || (XmVersion == 2002 && XmUPDATE_LEVEL < 3)
     if (state && XtIsRealized(widget))
     {
         /* 
@@ -2144,9 +2165,100 @@ void RadioButtonChangeStateNotified(Widget widget, Boolean state)
         XtCallActionProc(widget, "Select", &ev, NULL, 0);
         XtCallActionProc(widget, "Disarm", &ev, NULL, 0);
     }
-#endif /* XmVersion >= 2000 */
+#endif /* XmVersion == 2001 || ... */
 #endif /* LESSTIF_VERSION */
     
     /* This is sufficient on non-OM platforms */
-    XmToggleButtonSetState(widget, state, True);
+    XmToggleButtonSetState(widget, state, notify);
+}
+
+/* Workaround for bug in OpenMotif 2.1 and 2.2.  If you have an active tear-off
+** menu from a TopLevelShell that is a child of an ApplicationShell, and then 
+** close the parent window, Motif crashes.  The problem doesn't
+** happen if you close the tear-offs first, so, we programatically close them
+** before destroying the shell widget. 
+*/
+void CloseAllPopupsFor(Widget shell)
+{
+#ifndef LESSTIF_VERSION
+    /* Doesn't happen in LessTif.  The tear-off menus are popup-children of
+     * of the TopLevelShell there, which just works.  Motif wants to make
+     * them popup-children of the ApplicationShell, where it seems to get
+     * into trouble. */
+
+    Widget app = XtParent(shell);
+    int i;
+
+    for (i = 0; i < app->core.num_popups; i++) {
+        Widget pop = app->core.popup_list[i];
+        Widget shellFor;
+
+        XtVaGetValues(pop, XtNtransientFor, &shellFor, NULL);
+        if (shell == shellFor)
+            _XmDismissTearOff(pop, NULL, NULL);
+    }
+#endif
+}
+
+static long queryDesktop(Display *display, Window window, Atom deskTopAtom)
+{
+    long deskTopNumber = 0;
+    Atom actualType;
+    int actualFormat;
+    unsigned long nItems, bytesAfter;
+    unsigned char *prop;
+
+    if (XGetWindowProperty(display, window, deskTopAtom, 0, 1,
+           False, AnyPropertyType, &actualType, &actualFormat, &nItems,
+           &bytesAfter, &prop) != Success) {
+        return -1; /* Property not found */
+    }
+
+    if (actualType == None) {
+        return -1; /* Property does not exist */
+    }
+
+    if (actualFormat != 32 || nItems != 1) {
+        XFree((char*)prop);
+        return -1; /* Wrong format */
+    }
+
+    deskTopNumber = *(long*)prop;
+    XFree((char*)prop);
+    return deskTopNumber;
+}
+
+/*
+** Returns the current desktop number, or -1 if no desktop information
+** is available.
+*/
+long QueryCurrentDesktop(Display *display, Window rootWindow)
+{
+    static Atom currentDesktopAtom = (Atom)-1;
+
+    if (currentDesktopAtom == (Atom)-1)
+        currentDesktopAtom = XInternAtom(display, "_NET_CURRENT_DESKTOP", True);
+
+    if (currentDesktopAtom != None)
+        return queryDesktop(display, rootWindow, currentDesktopAtom);
+
+    return -1; /* No desktop information */
+}
+
+/*
+** Returns the number of the desktop the given shell window is currently on,
+** or -1 if no desktop information is available.  Note that windows shown
+** on all desktops (sometimes called sticky windows) should return 0xFFFFFFFF.
+*/
+long QueryDesktop(Display *display, Widget shell)
+{
+    static Atom wmDesktopAtom = (Atom)-1;
+
+    if (wmDesktopAtom == (Atom)-1)
+        wmDesktopAtom = XInternAtom(display, "_NET_WM_DESKTOP", True);
+
+    if (wmDesktopAtom != None)
+        return queryDesktop(display, XtWindow(shell), wmDesktopAtom);
+
+    return -1;  /* No desktop information */
 }
